package Test::Mojo::Role::Slovo;

BEGIN {
  binmode STDOUT => ':utf8';
  binmode STDERR => ':utf8';
}
use Mojo::Base -role, -signatures;
use feature qw(lexical_subs unicode_strings);
## no critic qw(TestingAndDebugging::ProhibitNoWarnings)
no warnings "experimental::lexical_subs";
use Test::More;
use Mojo::File qw(path tempdir);
use Mojo::ByteStream 'b';
use Mojo::Util qw(encode sha1_sum);
use FindBin;
my $default_from   = path($FindBin::Bin)->sibling('./');
my $random_tempdir = tempdir('slovoXXXX');

# Class method
# Install the app  from a path to a temporary path. Creates a log directory in
# installation directory to hide log output from screen and put it to slovo.log
# if $to_tempdir equals $random_tempdir.
# You can pass '/tmp/slovo' after $from. The tmp/slovo will not be
# automatically deleted and you can debug the installed application.
sub install ($class, $from = $default_from, $to_tempdir = $random_tempdir) {
  my $MOJO_HOME = path($to_tempdir);

  # idempotent
  $MOJO_HOME->remove_tree->make_path({mode => 0700});
  ok(-d $MOJO_HOME, "created $MOJO_HOME");
  $MOJO_HOME->child('log')->make_path({mode => 0700})
    if $to_tempdir eq $random_tempdir;
  path($from, 'lib')->list_tree({dir => 1})->each(
    sub ($f, $i) {
      $f =~ /\.sqlite$/ && return;    #do not copy existing database
      my $new = $MOJO_HOME->child($f->to_rel);
      (-d $f) && $new->make_path({mode => 0700});
      (-f $f) && $f->copy_to($new);
    }
  );
  unshift @INC, path($to_tempdir, 'lib')->to_string;
  return $class;
}

# use this method for the side effect of having a logged in user
sub login_ok ($t, $login_name, $login_password) {
  subtest login_ok => sub {
    my $login_url = $t->app->url_for('sign_in')->to_string;

    $t->get_ok('/Ꙋправленѥ')->status_is(302)
      ->header_is(Location => $login_url, 'Location is /входъ');
    $t->get_ok('/входъ')->status_is(200)->text_is('head title' => 'Входъ');

#get the csrf field
    my $csrf_token
      = $t->tx->res->dom->at('#sign_in [name="csrf_token"]')->{value};
    my $form = {
          login_name => $login_name,
          csrf_token => $csrf_token,
          digest     => sha1_sum(
            $csrf_token . sha1_sum(encode('utf8', "$login_name$login_password"))
          ),
    };

    $t->post_ok($login_url, {} => form => $form)->status_is(302)
      ->header_is(Location => '/', 'Location: /')
      ->content_is('', 'empty content');
  };
  return $t;
}

sub login ($t, $login_name, $login_password) {
  my $login_url = $t->app->url_for('sign_in')->to_string;
  my $csrf_token
    = $t->ua->get($login_url)->res->dom->at('#sign_in [name="csrf_token"]')
    ->{value};
  my $form = {
          login_name => $login_name,
          csrf_token => $csrf_token,
          digest     => sha1_sum(
            $csrf_token . sha1_sum(encode('utf8', "$login_name$login_password"))
          ),
  };
  return $t->ua->post($login_url => {} => form => $form)->res->body eq '';
}
1;
